package com.malk.utils;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import lombok.SneakyThrows;
import org.springframework.core.io.ClassPathResource;

import javax.annotation.Nullable;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;

public abstract class UtilFile {

    /**
     * 匹配路径: 自动追加年月日作为目录
     * -
     * 文件上传存储,必须是绝对路径. 日志的配置地址可以是相对路径
     */
    public static File mkdirIfNot(@Nullable String fileName, @Nullable String dirName) {
        String date = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
        String outputFile = dirName + "/" + date + "/" + fileName;
        File file = new File(outputFile);
        // 创建 mkdirIfNot
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        return file;
    }

    /**
     * 文件转化为byte字节数组
     */
    public static byte[] fileConvertToByteArray(File file) {
        byte[] data = null;
        try {
            FileInputStream fis = new FileInputStream(file);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int len;
            byte[] buffer = new byte[1024];
            while ((len = fis.read(buffer)) != -1) {
                baos.write(buffer, 0, len);
            }
            data = baos.toByteArray();
            fis.close();
            baos.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return data;
    }

    /**
     * 本地路径转json string
     */
    @SneakyThrows
    public static String readJsonStringFromFile(String path) {
        File jsonFile = new File(path);
        FileReader fileReader = new FileReader(jsonFile);

        Reader reader = new InputStreamReader(new FileInputStream(jsonFile), "utf-8");
        int ch = 0;
        StringBuffer sb = new StringBuffer();
        while ((ch = reader.read()) != -1) {
            sb.append((char) ch);
        }
        fileReader.close();
        reader.close();
        return sb.toString();
    }

    /**
     * 项目包文件, 读取 [服务Excel导出]
     * [ppExt]
     * 1. ClassPathResource, 需要打包后才能访问到. 识别不是架包内内容
     * 2. 路径若 WebConfiguration 配置, 可使用配置别名
     * 3. 若是读取本地json, 文件不能使用 .josn 后缀, 会被转义导致解析异常
     */
    @SneakyThrows
    public static InputStream readPackageResource(String path) {
        ClassPathResource classPathResource = new ClassPathResource(path);
        InputStream inputStream = classPathResource.getInputStream();
        return inputStream;
    }

    /**
     * 项目包文件, 读取 [服务读取本地json string]
     */
    public static String readPackageResourcePath(String path) {
        return JSONUtil.class.getClassLoader().getResource(path).getPath();
    }

    /**
     * 项目包文件, 读取 [服务读取本地json]
     */
    public static Object readJsonObjectFromFile(String path) {
        String json = readJsonStringFromFile(readPackageResourcePath(path));
        return JSON.parse(json);
    }
}
